# -*- coding: utf-8 -*-
from django.core.management.commands.makemigrations import Command as MakeMigrationCommand
from django.db.migrations.loader import MigrationLoader
from django.utils import timezone

from thx.role import __version__


class Command(MakeMigrationCommand):
    help = "Make roles migrations"

    def handle(self, *args, **options):
        options['empty'] = True
        options['name'] = "update_roles"
        super().handle(*args, **options)
        app_label = args[0]

        loader = MigrationLoader(None, ignore_no_migrations=True)

        migration_name = None
        role_migration_name = None
        leaves = loader.graph.leaf_nodes()
        for leave in leaves:
            if leave[0] == app_label:
                migration_name = leave[1]
                if role_migration_name:
                    break
            if leave[0] == "role":
                role_migration_name = leave[1]
                if migration_name:
                    break

        app_label_dependency = loader.graph.forwards_plan(('account', migration_name))[-2]
        role_dependency = ('role', role_migration_name)
        dependencies = [app_label_dependency, role_dependency]

        if migration_name:
            self.write_update_roles_migrations(app_label, migration_name, dependencies)

    def write_update_roles_migrations(self, app_label, migration_name, dependencies):
        with open(f"apps/{app_label}/migrations/{migration_name}.py", "w") as f:
            f.write(self.get_migration_template(dependencies))

    def get_migration_template(self, dependencies):
        date = timezone.now().strftime("%Y-%m-%d %H:%M")
        return f"""# Generated by thx-role {__version__} on {date}

from django.core.management import call_command
from django.db import migrations


def update_roles(apps, schema_editor):
    call_command("populate_roles")
    Profile = apps.get_model("role", "Profile")  # noqa
    Role = apps.get_model("role", "Role")  # noqa
        
    # your code goes here...


def reverse_update_roles(apps, schema_editor):
    Profile = apps.get_model("role", "Profile")  # noqa
    Role = apps.get_model("role", "Role")  # noqa
    
    # your code goes here...


class Migration(migrations.Migration):

    dependencies = [
        {dependencies[0]},
        {dependencies[1]},
    ]

    operations = [
        migrations.RunPython(code=update_roles, reverse_code=reverse_update_roles)
    ]        
"""
